@@ -37,6 +37,7 @@
 #endif
 
 #include <math.h>
+#include <float.h>
 
 #include "gl_opengl.h"
 #include "doomstat.h"
@@ -57,205 +58,105 @@
 #include "m_bbox.h"
 #include "d_main.h"
 
+#include "dsda/id_list.h"
 #include "dsda/input.h"
 #include "dsda/map_format.h"
 #include "dsda/messenger.h"
 #include "dsda/settings.h"
 #include "dsda/stretch.h"
+#include "dsda/utility.h"
 
-//jff 1/7/98 default automap colors added
-int mapcolor_back;    // map background
-int mapcolor_grid;    // grid lines color
-int mapcolor_wall;    // normal 1s wall color
-int mapcolor_fchg;    // line at floor height change color
-int mapcolor_cchg;    // line at ceiling height change color
-int mapcolor_clsd;    // line at sector with floor=ceiling color
-int mapcolor_rkey;    // red key color
-int mapcolor_bkey;    // blue key color
-int mapcolor_ykey;    // yellow key color
-int mapcolor_rdor;    // red door color  (diff from keys to allow option)
-int mapcolor_bdor;    // blue door color (of enabling one but not other )
-int mapcolor_ydor;    // yellow door color
-int mapcolor_tele;    // teleporter line color
-int mapcolor_secr;    // secret sector boundary color
-int mapcolor_revsecr; // revealed secret sector boundary color
-int mapcolor_exit;    // jff 4/23/98 add exit line color
-int mapcolor_unsn;    // computer map unseen line color
-int mapcolor_flat;    // line with no floor/ceiling changes
-int mapcolor_sprt;    // general sprite color
-int mapcolor_item;    // item sprite color
-int mapcolor_frnd;    // friendly sprite color
-int mapcolor_enemy;   // enemy sprite color
-int mapcolor_hair;    // crosshair color
-int mapcolor_sngl;    // single player arrow color
-int mapcolor_plyr[4] = { 112, 96, 64, 176 }; // colors for player arrows in multiplayer
+mapcolor_t mapcolor = {
+  .plyr = { 112, 96, 64, 176 },
+};
 
-static int heretic_mapcolor_back = 0;
-static int heretic_mapcolor_grid = 5 * 8;
-static int heretic_mapcolor_wall = 104;
-static int heretic_mapcolor_fchg = 88;
-static int heretic_mapcolor_cchg = 10 * 8;
-static int heretic_mapcolor_clsd;
-static int heretic_mapcolor_rkey = 220;
-static int heretic_mapcolor_bkey = 197;
-static int heretic_mapcolor_ykey = 144;
-static int heretic_mapcolor_rdor = 220;
-static int heretic_mapcolor_bdor = 197;
-static int heretic_mapcolor_ydor = 144;
-static int heretic_mapcolor_tele = 88;
-static int heretic_mapcolor_secr = 88;
-static int heretic_mapcolor_revsecr = 88;
-static int heretic_mapcolor_exit;
-static int heretic_mapcolor_unsn = 5 * 8 + 3;
-static int heretic_mapcolor_flat;
-static int heretic_mapcolor_sprt = 224;
-static int heretic_mapcolor_item = 144;
-static int heretic_mapcolor_frnd = 224;
-static int heretic_mapcolor_enemy = 160;
-static int heretic_mapcolor_hair = 5 * 8;
-static int heretic_mapcolor_sngl = 4 * 8;
-static int heretic_mapcolor_plyr[4] = { 220, 144, 150, 197 };
+mapcolor_t mapcolor_heretic = {
+  .back = 0,
+  .grid = 5 * 8,
+  .wall = 104,
+  .fchg = 88,
+  .cchg = 10 * 8,
+  // .clsd
+  .rkey = 220,
+  .bkey = 197,
+  .ykey = 144,
+  .rdor = 220,
+  .bdor = 197,
+  .ydor = 144,
+  .tele = 88,
+  .secr = 88,
+  .revsecr = 88,
+  // .exit
+  .unsn = 5 * 8 + 3,
+  // .flat
+  .sprt = 224,
+  .item = 144,
+  .frnd = 224,
+  .enemy = 160,
+  .hair = 5 * 8,
+  .sngl = 4 * 8,
+  .plyr = { 220, 144, 150, 197 },
+  .trail_1 = 33,
+  .trail_2 = 15,
+};
 
-static int hexen_mapcolor_back = 0;
-static int hexen_mapcolor_grid = 5 * 8;
-static int hexen_mapcolor_wall = 12 * 8;
-static int hexen_mapcolor_fchg = 14 * 8;
-static int hexen_mapcolor_cchg = 10 * 8;
-static int hexen_mapcolor_clsd;
-static int hexen_mapcolor_rkey;
-static int hexen_mapcolor_bkey;
-static int hexen_mapcolor_ykey;
-static int hexen_mapcolor_rdor = 198;
-static int hexen_mapcolor_bdor = 198;
-static int hexen_mapcolor_ydor = 198;
-static int hexen_mapcolor_tele = 157;
-static int hexen_mapcolor_secr;
-static int hexen_mapcolor_revsecr;
-static int hexen_mapcolor_exit = 177;
-static int hexen_mapcolor_unsn = 5 * 8 + 3;
-static int hexen_mapcolor_flat;
-static int hexen_mapcolor_sprt = 216;
-static int hexen_mapcolor_item = 230;
-static int hexen_mapcolor_frnd = 216;
-static int hexen_mapcolor_enemy = 176;
-static int hexen_mapcolor_hair = 5 * 8;
-static int hexen_mapcolor_sngl = 4 * 8;
-static int hexen_mapcolor_plyr[8] = { 157, 177, 137, 198, 215, 32, 106, 234 };
+mapcolor_t mapcolor_hexen = {
+  .back = 0,
+  .grid = 5 * 8,
+  .wall = 12 * 8,
+  .fchg = 14 * 8,
+  .cchg = 10 * 8,
+  // .clsd
+  // .rkey
+  // .bkey
+  // .ykey
+  .rdor = 198,
+  .bdor = 198,
+  .ydor = 198,
+  .tele = 157,
+  // .secr
+  // .revsecr
+  .exit = 177,
+  .unsn = 5 * 8 + 3,
+  // .flat
+  .sprt = 216,
+  .item = 230,
+  .frnd = 216,
+  .enemy = 176,
+  .hair = 5 * 8,
+  .sngl = 4 * 8,
+  .plyr = { 157, 177, 137, 198, 215, 32, 106, 234 },
+  .trail_1 = 33,
+  .trail_2 = 15,
+};
 
-static int* mapcolor_back_p;
-static int* mapcolor_grid_p;
-static int* mapcolor_wall_p;
-static int* mapcolor_fchg_p;
-static int* mapcolor_cchg_p;
-static int* mapcolor_clsd_p;
-static int* mapcolor_rkey_p;
-static int* mapcolor_bkey_p;
-static int* mapcolor_ykey_p;
-static int* mapcolor_rdor_p;
-static int* mapcolor_bdor_p;
-static int* mapcolor_ydor_p;
-static int* mapcolor_tele_p;
-static int* mapcolor_secr_p;
-static int* mapcolor_revsecr_p;
-static int* mapcolor_exit_p;
-static int* mapcolor_unsn_p;
-static int* mapcolor_flat_p;
-static int* mapcolor_sprt_p;
-static int* mapcolor_item_p;
-static int* mapcolor_frnd_p;
-static int* mapcolor_enemy_p;
-static int* mapcolor_hair_p;
-static int* mapcolor_sngl_p;
-static int* mapcolor_plyr_p;
+static mapcolor_t *mapcolor_p;
 
 static void AM_SetColors(void)
 {
-  if (heretic)
-  {
-    mapcolor_back_p = &heretic_mapcolor_back;
-    mapcolor_grid_p = &heretic_mapcolor_grid;
-    mapcolor_wall_p = &heretic_mapcolor_wall;
-    mapcolor_fchg_p = &heretic_mapcolor_fchg;
-    mapcolor_cchg_p = &heretic_mapcolor_cchg;
-    mapcolor_clsd_p = &heretic_mapcolor_clsd;
-    mapcolor_rkey_p = &heretic_mapcolor_rkey;
-    mapcolor_bkey_p = &heretic_mapcolor_bkey;
-    mapcolor_ykey_p = &heretic_mapcolor_ykey;
-    mapcolor_rdor_p = &heretic_mapcolor_rdor;
-    mapcolor_bdor_p = &heretic_mapcolor_bdor;
-    mapcolor_ydor_p = &heretic_mapcolor_ydor;
-    mapcolor_tele_p = &heretic_mapcolor_tele;
-    mapcolor_secr_p = &heretic_mapcolor_secr;
-    mapcolor_revsecr_p = &heretic_mapcolor_revsecr;
-    mapcolor_exit_p = &heretic_mapcolor_exit;
-    mapcolor_unsn_p = &heretic_mapcolor_unsn;
-    mapcolor_flat_p = &heretic_mapcolor_flat;
-    mapcolor_sprt_p = &heretic_mapcolor_sprt;
-    mapcolor_item_p = &heretic_mapcolor_item;
-    mapcolor_frnd_p = &heretic_mapcolor_frnd;
-    mapcolor_enemy_p = &heretic_mapcolor_enemy;
-    mapcolor_hair_p = &heretic_mapcolor_hair;
-    mapcolor_sngl_p = &heretic_mapcolor_sngl;
-    mapcolor_plyr_p = heretic_mapcolor_plyr;
-  }
-  else if (hexen)
-  {
-    mapcolor_back_p = &hexen_mapcolor_back;
-    mapcolor_grid_p = &hexen_mapcolor_grid;
-    mapcolor_wall_p = &hexen_mapcolor_wall;
-    mapcolor_fchg_p = &hexen_mapcolor_fchg;
-    mapcolor_cchg_p = &hexen_mapcolor_cchg;
-    mapcolor_clsd_p = &hexen_mapcolor_clsd;
-    mapcolor_rkey_p = &hexen_mapcolor_rkey;
-    mapcolor_bkey_p = &hexen_mapcolor_bkey;
-    mapcolor_ykey_p = &hexen_mapcolor_ykey;
-    mapcolor_rdor_p = &hexen_mapcolor_rdor;
-    mapcolor_bdor_p = &hexen_mapcolor_bdor;
-    mapcolor_ydor_p = &hexen_mapcolor_ydor;
-    mapcolor_tele_p = &hexen_mapcolor_tele;
-    mapcolor_secr_p = &hexen_mapcolor_secr;
-    mapcolor_revsecr_p = &hexen_mapcolor_revsecr;
-    mapcolor_exit_p = &hexen_mapcolor_exit;
-    mapcolor_unsn_p = &hexen_mapcolor_unsn;
-    mapcolor_flat_p = &hexen_mapcolor_flat;
-    mapcolor_sprt_p = &hexen_mapcolor_sprt;
-    mapcolor_item_p = &hexen_mapcolor_item;
-    mapcolor_frnd_p = &hexen_mapcolor_frnd;
-    mapcolor_enemy_p = &hexen_mapcolor_enemy;
-    mapcolor_hair_p = &hexen_mapcolor_hair;
-    mapcolor_sngl_p = &hexen_mapcolor_sngl;
-    mapcolor_plyr_p = hexen_mapcolor_plyr;
-  }
-  else
-  {
-    mapcolor_back_p = &mapcolor_back;
-    mapcolor_grid_p = &mapcolor_grid;
-    mapcolor_wall_p = &mapcolor_wall;
-    mapcolor_fchg_p = &mapcolor_fchg;
-    mapcolor_cchg_p = &mapcolor_cchg;
-    mapcolor_clsd_p = &mapcolor_clsd;
-    mapcolor_rkey_p = &mapcolor_rkey;
-    mapcolor_bkey_p = &mapcolor_bkey;
-    mapcolor_ykey_p = &mapcolor_ykey;
-    mapcolor_rdor_p = &mapcolor_rdor;
-    mapcolor_bdor_p = &mapcolor_bdor;
-    mapcolor_ydor_p = &mapcolor_ydor;
-    mapcolor_tele_p = &mapcolor_tele;
-    mapcolor_secr_p = &mapcolor_secr;
-    mapcolor_revsecr_p = &mapcolor_revsecr;
-    mapcolor_exit_p = &mapcolor_exit;
-    mapcolor_unsn_p = &mapcolor_unsn;
-    mapcolor_flat_p = &mapcolor_flat;
-    mapcolor_sprt_p = &mapcolor_sprt;
-    mapcolor_item_p = &mapcolor_item;
-    mapcolor_frnd_p = &mapcolor_frnd;
-    mapcolor_enemy_p = &mapcolor_enemy;
-    mapcolor_hair_p = &mapcolor_hair;
-    mapcolor_sngl_p = &mapcolor_sngl;
-    mapcolor_plyr_p = mapcolor_plyr;
-  }
+  mapcolor_p = hexen   ? &mapcolor_hexen   :
+               heretic ? &mapcolor_heretic :
+                         &mapcolor;
 }
 
+typedef struct
+{
+  mpoint_t a, b;
+} mline_t;
+
+typedef struct
+{
+  int tag;
+  fixed_t x, y;
+  sector_t *sec;
+  line_t *line;
+  mline_t *connections;
+  int connection_count;
+  int connection_max;
+} highlight_t;
+
+static highlight_t highlight;
+
 static int map_blinking_locks;
 static int map_secret_after;
 static int map_grid_size;
@@ -296,11 +197,6 @@ static map_things_appearance_t map_things_appearance;
 #define CXMTOF_F(x)  ((float)f_x + MTOF_F((x)-m_x))
 #define CYMTOF_F(y)  ((float)f_y + (f_h - MTOF_F((y)-m_y)))
 
-typedef struct
-{
-    mpoint_t a, b;
-} mline_t;
-
 #define R ((8*PLAYERRADIUS)/7)
 mline_t hexen_player_arrow[] = {
   { { -R+R/4, 0 }, { 0, 0} }, // center line.
@@ -442,6 +338,20 @@ markpoint_t *markpoints = NULL;    // where the points are
 int markpointnum = 0; // next point to be assigned (also number of points now)
 int markpointnum_max = 0;       // killough 2/22/98
 
+typedef struct
+{
+  fixed_t x;
+  fixed_t y;
+} trailpoint_t;
+
+#define TRAIL_SIZE 350 // ten seconds
+static trailpoint_t player_trail[TRAIL_SIZE];
+static int trail_index;
+static int trail_size;
+static int trail_size_max;
+
+map_trail_mode_t map_trail_mode;
+
 am_frame_t am_frame;
 
 array_t map_lines;
@@ -723,6 +633,14 @@ void AM_SetPosition(void)
   }
 }
 
+void AM_initPlayerTrail(void)
+{
+  trail_index = -1;
+  trail_size = 0;
+  trail_size_max = dsda_IntConfig(dsda_config_map_trail_size);
+  map_trail_mode = dsda_IntConfig(dsda_config_map_trail_mode);
+}
+
 //
 // AM_initVariables()
 //
@@ -736,6 +654,8 @@ static void AM_initVariables(void)
 {
   int pnum;
 
+  AM_initPlayerTrail();
+
   if (hexen)
   {
     numplyrlines = HEXEN_NUMPLYRLINES;
@@ -784,6 +704,14 @@ void AM_SetResolution(void)
   AM_SetScale();
 }
 
+static void AM_ResetTagHighlight(void)
+{
+  Z_Free(highlight.connections);
+  ZERO_DATA(highlight);
+  highlight.x = INT_MIN;
+  highlight.y = INT_MIN;
+}
+
 //
 // AM_clearMarks()
 //
@@ -794,6 +722,9 @@ void AM_SetResolution(void)
 //
 void AM_clearMarks(void)
 {
+  AM_initPlayerTrail();
+  AM_ResetTagHighlight();
+
   markpointnum = 0;
 }
 
@@ -917,6 +848,150 @@ static void AM_maxOutWindowScale(void)
   AM_activateNewScale();
 }
 
+static line_t *AM_ClosestLine(fixed_t x, fixed_t y, sector_t *sec)
+{
+  int i;
+  line_t *closest_line = NULL;
+  double closest_distance = DBL_MAX;
+
+  for (i = 0; i < sec->linecount; ++i)
+  {
+    line_t *line;
+    double dist;
+
+    line = sec->lines[i];
+    dist = dsda_DistancePointToLine(
+      line->v1->x >> FRACTOMAPBITS, line->v1->y >> FRACTOMAPBITS,
+      line->v2->x >> FRACTOMAPBITS, line->v2->y >> FRACTOMAPBITS,
+      x, y
+    );
+
+    if (dist < closest_distance)
+    {
+      closest_line = line;
+      closest_distance = dist;
+    }
+  }
+
+  return closest_line;
+}
+
+static void AM_AddHighlightConnection(mpoint_t a, mpoint_t b)
+{
+  if (!highlight.connection_max)
+  {
+    highlight.connection_max = 4;
+    highlight.connections =
+      Z_Realloc(highlight.connections,
+                highlight.connection_max * sizeof(*highlight.connections));
+  }
+
+  if (highlight.connection_count == highlight.connection_max)
+  {
+    highlight.connection_max *= 2;
+    highlight.connections =
+      Z_Realloc(highlight.connections,
+                highlight.connection_max * sizeof(*highlight.connections));
+  }
+
+  highlight.connections[highlight.connection_count].a = a;
+  highlight.connections[highlight.connection_count].b = b;
+  ++highlight.connection_count;
+}
+
+static void AM_HighlightByTag(void)
+{
+  fixed_t x, y;
+  sector_t *sec;
+  line_t *line;
+  dboolean repeat;
+
+  x = m_x + m_w / 2;
+  y = m_y + m_h / 2;
+
+  repeat = (x == highlight.x && y == highlight.y);
+  highlight.x = x;
+  highlight.y = y;
+
+  sec = R_PointInSector(x << FRACTOMAPBITS, y << FRACTOMAPBITS);
+  line = AM_ClosestLine(x, y, sec);
+
+  if (!repeat || (!highlight.sec && !highlight.line))
+  {
+    highlight.sec = sec;
+    highlight.line = NULL;
+    highlight.tag = sec->tag;
+
+    doom_printf("Highlight sector %d, tag %d\n", highlight.sec->iSectorID, sec->tag);
+  }
+  else if (highlight.sec)
+  {
+    highlight.sec = NULL;
+    highlight.line = line;
+    highlight.tag = line->tag;
+
+    doom_printf("Highlight line %d, tag %d\n", highlight.line->iLineID, line->tag);
+  }
+  else
+  {
+    highlight.line = NULL;
+    highlight.tag = 0;
+
+    doom_printf("Highlight nothing\n");
+  }
+
+  Z_Free(highlight.connections);
+  highlight.connections = NULL;
+  highlight.connection_count = 0;
+  highlight.connection_max = 0;
+
+  if (highlight.tag)
+  {
+    const int *id_p;
+    mpoint_t origin;
+    mpoint_t destination;
+
+    if (highlight.line)
+    {
+      sector_t *sec;
+
+      R_LineCenter(&origin.x, &origin.y, highlight.line);
+      origin.x >>= FRACTOMAPBITS;
+      origin.y >>= FRACTOMAPBITS;
+
+      FIND_SECTORS(id_p, highlight.tag)
+      {
+        sec = &sectors[*id_p];
+
+        R_SectorCenter(&destination.x, &destination.y, sec);
+        destination.x >>= FRACTOMAPBITS;
+        destination.y >>= FRACTOMAPBITS;
+
+        AM_AddHighlightConnection(origin, destination);
+      }
+    }
+    else
+    {
+      line_t *line;
+
+      R_SectorCenter(&origin.x, &origin.y, highlight.sec);
+      origin.x >>= FRACTOMAPBITS;
+      origin.y >>= FRACTOMAPBITS;
+
+      FIND_LINES(id_p, highlight.tag)
+      {
+        line = &lines[*id_p];
+
+        R_LineCenter(&destination.x, &destination.y, line);
+        destination.x >>= FRACTOMAPBITS;
+        destination.y >>= FRACTOMAPBITS;
+
+        AM_AddHighlightConnection(origin, destination);
+      }
+    }
+  }
+}
+
 //
 // AM_Responder()
 //
@@ -1099,6 +1174,15 @@ dboolean AM_Responder
     if (leveltime != zoom_leveltime)
       AM_StopZooming();
   }
+  else if (dsda_InputActivated(dsda_input_map_highlight_by_tag))
+  {
+    if (!dsda_RevealAutomap())
+      doom_printf("Highlight requires iddt");
+    else
+      AM_HighlightByTag();
+
+    return true;
+  }
 
   return false;
 }
@@ -1515,12 +1599,12 @@ static void AM_drawGrid(int color)
 
 static dboolean AM_DrawHiddenSecrets(void)
 {
-  return !!(*mapcolor_secr_p) && !map_secret_after;
+  return !!mapcolor_p->secr && !map_secret_after;
 }
 
 static dboolean AM_DrawRevealedSecrets(void)
 {
-  return !!(*mapcolor_revsecr_p);
+  return !!mapcolor_p->revsecr;
 }
 
 //
@@ -1566,12 +1650,12 @@ static automap_style_t AM_wallStyle(int i)
       return ams_invisible;
 
     if (
-      ((*mapcolor_bdor_p) || (*mapcolor_ydor_p) || (*mapcolor_rdor_p)) &&
+      (mapcolor_p->bdor || mapcolor_p->ydor || mapcolor_p->rdor) &&
       !(lines[i].flags & ML_SECRET) && dsda_DoorType(i) != -1
     )
       return ams_locked;
 
-    if ((*mapcolor_exit_p) && dsda_IsExitLine(i))
+    if (mapcolor_p->exit && dsda_IsExitLine(i))
       return ams_exit;
 
     if (!lines[i].backsector) // 1-sided
@@ -1585,7 +1669,7 @@ static automap_style_t AM_wallStyle(int i)
     }
     else // 2-sided
     {
-      if ((*mapcolor_tele_p) && !(lines[i].flags & ML_SECRET) && dsda_IsTeleportLine(i))
+      if (mapcolor_p->tele && !(lines[i].flags & ML_SECRET) && dsda_IsTeleportLine(i))
       {
         return ams_teleport;
       }
@@ -1594,7 +1678,7 @@ static automap_style_t AM_wallStyle(int i)
         return ams_one_sided;
       }
       else if (
-        (*mapcolor_clsd_p) &&
+        mapcolor_p->clsd &&
         !(lines[i].flags & ML_SECRET) &&
         ((lines[i].backsector->floorheight==lines[i].backsector->ceilingheight) ||
         (lines[i].frontsector->floorheight==lines[i].frontsector->ceilingheight))
@@ -1626,7 +1710,7 @@ static automap_style_t AM_wallStyle(int i)
       {
         return ams_ceiling_diff;
       }
-      else if ((*mapcolor_flat_p) && dsda_RevealAutomap())
+      else if (mapcolor_p->flat && dsda_RevealAutomap())
       {
         return ams_two_sided;
       }
@@ -1638,7 +1722,7 @@ static automap_style_t AM_wallStyle(int i)
     {
       if
       (
-        (*mapcolor_flat_p) ||
+        mapcolor_p->flat ||
         !lines[i].backsector ||
         lines[i].backsector->floorheight != lines[i].frontsector->floorheight ||
         lines[i].backsector->ceilingheight != lines[i].frontsector->ceilingheight
@@ -1696,70 +1780,97 @@ static void AM_drawWalls(void)
       case ams_locked:
         if (hide_locks)
         {
-          AM_drawMline(&l, *mapcolor_grid_p);
+          AM_drawMline(&l, mapcolor_p->grid);
           continue;
         }
 
         switch (dsda_DoorType(i))
         {
           case 0: // red
-            AM_drawMline(&l, (*mapcolor_rdor_p)? (*mapcolor_rdor_p) : (*mapcolor_cchg_p));
+            AM_drawMline(&l, mapcolor_p->rdor? mapcolor_p->rdor : mapcolor_p->cchg);
             continue;
           case 1: // blue
-            AM_drawMline(&l, (*mapcolor_bdor_p)? (*mapcolor_bdor_p) : (*mapcolor_cchg_p));
+            AM_drawMline(&l, mapcolor_p->bdor? mapcolor_p->bdor : mapcolor_p->cchg);
             continue;
           case 2: // yellow
-            AM_drawMline(&l, (*mapcolor_ydor_p)? (*mapcolor_ydor_p) : (*mapcolor_cchg_p));
+            AM_drawMline(&l, mapcolor_p->ydor? mapcolor_p->ydor : mapcolor_p->cchg);
             continue;
           default:
-            AM_drawMline(&l, (*mapcolor_clsd_p)? (*mapcolor_clsd_p) : (*mapcolor_cchg_p));
+            AM_drawMline(&l, mapcolor_p->clsd? mapcolor_p->clsd : mapcolor_p->cchg);
             continue;
         }
 
       case ams_exit:
-        AM_drawMline(&l, (*mapcolor_exit_p));
+        AM_drawMline(&l, mapcolor_p->exit);
         continue;
 
       case ams_one_sided:
-        AM_drawMline(&l, (*mapcolor_wall_p));
+        AM_drawMline(&l, mapcolor_p->wall);
         continue;
 
       case ams_secret:
       case ams_unseen_secret:
-        AM_drawMline(&l, (*mapcolor_secr_p));
+        AM_drawMline(&l, mapcolor_p->secr);
         continue;
 
       case ams_revealed_secret:
-        AM_drawMline(&l, (*mapcolor_revsecr_p));
+        AM_drawMline(&l, mapcolor_p->revsecr);
         continue;
 
       case ams_teleport:
-        AM_drawMline(&l, (*mapcolor_tele_p));
+        AM_drawMline(&l, mapcolor_p->tele);
         continue;
 
       case ams_closed_door:
-        AM_drawMline(&l, (*mapcolor_clsd_p));
+        AM_drawMline(&l, mapcolor_p->clsd);
         continue;
 
       case ams_floor_diff:
-        AM_drawMline(&l, (*mapcolor_fchg_p));
+        AM_drawMline(&l, mapcolor_p->fchg);
         continue;
 
       case ams_ceiling_diff:
-        AM_drawMline(&l, (*mapcolor_cchg_p));
+        AM_drawMline(&l, mapcolor_p->cchg);
         continue;
 
       case ams_two_sided:
-        AM_drawMline(&l, (*mapcolor_flat_p));
+        AM_drawMline(&l, mapcolor_p->flat);
         continue;
 
       case ams_unseen:
-        AM_drawMline(&l, (*mapcolor_unsn_p));
+        AM_drawMline(&l, mapcolor_p->unsn);
         continue;
     }
   }
 }
 
+static void AM_DrawConnections(void)
+{
+  int i;
+  mline_t l;
+
+  if (!dsda_RevealAutomap())
+    return;
+
+  for (i = 0; i < highlight.connection_count; ++i)
+  {
+    l = highlight.connections[i];
+
+    if (automap_rotate)
+    {
+      AM_rotatePoint(&l.a);
+      AM_rotatePoint(&l.b);
+    }
+    else
+    {
+      AM_SetMPointFloatValue(&l.a);
+      AM_SetMPointFloatValue(&l.b);
+    }
+
+    AM_drawMline(&l, mapcolor_p->exit);
+  }
+}
+
 //
 // AM_drawLineCharacter()
 //
@@ -1885,9 +1996,9 @@ static void AM_drawPlayers(void)
       AM_SetMPointFloatValue(&pt);
 
     if (dsda_RevealAutomap())
-      AM_drawLineCharacter(cheat_player_arrow, NUMCHEATPLYRLINES, scale, viewangle, (*mapcolor_sngl_p), pt.x, pt.y);
+      AM_drawLineCharacter(cheat_player_arrow, NUMCHEATPLYRLINES, scale, viewangle, mapcolor_p->sngl, pt.x, pt.y);
     else
-      AM_drawLineCharacter(player_arrow, numplyrlines, scale, viewangle, (*mapcolor_sngl_p), pt.x, pt.y);
+      AM_drawLineCharacter(player_arrow, numplyrlines, scale, viewangle, mapcolor_p->sngl, pt.x, pt.y);
     return;
   }
 
@@ -1908,7 +2019,7 @@ static void AM_drawPlayers(void)
 
       AM_drawLineCharacter (player_arrow, numplyrlines, scale, angle,
           p->powers[pw_invisibility] ? 246 /* *close* to black */
-          : mapcolor_plyr_p[i], //jff 1/6/98 use default color
+          : mapcolor_p->plyr[i], //jff 1/6/98 use default color
           pt.x, pt.y);
     }
   }
@@ -2000,7 +2111,7 @@ static void AM_ProcessNiceThing(mobj_t* mobj, angle_t angle, fixed_t x, fixed_t
   if (mobj->player)
   {
     player_t *p = mobj->player;
-    int color = mapcolor_plyr_p[p - players];
+    int color = mapcolor_p->plyr[p - players];
     const unsigned char *playpal = V_GetPlaypal();
 
     if ((deathmatch && !demoplayback) && p != plr)
@@ -2259,7 +2370,7 @@ static void AM_drawThings(void)
         AM_SetMPointFloatValue(&p);
 
       //jff 1/5/98 case over doomednum of thing being drawn
-      if ((*mapcolor_rkey_p) || (*mapcolor_ykey_p) || (*mapcolor_bkey_p))
+      if (mapcolor_p->rkey || mapcolor_p->ykey || mapcolor_p->bkey)
       {
         int color = -1;
 
@@ -2269,11 +2380,11 @@ static void AM_drawThings(void)
           {
             //jff 1/5/98 treat keys special
             case 73: //jff  red key
-              color = (*mapcolor_rkey_p) != -1? (*mapcolor_rkey_p) : (*mapcolor_sprt_p); break;
+              color = mapcolor_p->rkey != -1? mapcolor_p->rkey : mapcolor_p->sprt; break;
             case 80: //jff yellow key
-              color = (*mapcolor_ykey_p) != -1? (*mapcolor_ykey_p) : (*mapcolor_sprt_p); break;
+              color = mapcolor_p->ykey != -1? mapcolor_p->ykey : mapcolor_p->sprt; break;
             case 79: //jff blue key
-              color = (*mapcolor_bkey_p) != -1? (*mapcolor_bkey_p) : (*mapcolor_sprt_p); break;
+              color = mapcolor_p->bkey != -1? mapcolor_p->bkey : mapcolor_p->sprt; break;
           }
         }
         else
@@ -2282,11 +2393,11 @@ static void AM_drawThings(void)
           {
             //jff 1/5/98 treat keys special
             case 38: case 13: //jff  red key
-              color = (*mapcolor_rkey_p) != -1? (*mapcolor_rkey_p) : (*mapcolor_sprt_p); break;
+              color = mapcolor_p->rkey != -1? mapcolor_p->rkey : mapcolor_p->sprt; break;
             case 39: case 6: //jff yellow key
-              color = (*mapcolor_ykey_p) != -1? (*mapcolor_ykey_p) : (*mapcolor_sprt_p); break;
+              color = mapcolor_p->ykey != -1? mapcolor_p->ykey : mapcolor_p->sprt; break;
             case 40: case 5: //jff blue key
-              color = (*mapcolor_bkey_p) != -1? (*mapcolor_bkey_p) : (*mapcolor_sprt_p); break;
+              color = mapcolor_p->bkey != -1? mapcolor_p->bkey : mapcolor_p->sprt; break;
           }
         }
 
@@ -2302,11 +2413,11 @@ static void AM_drawThings(void)
       //jff previously entire code
       AM_drawLineCharacter(thintriangle_guy, NUMTHINTRIANGLEGUYLINES,
         scale, angle,
-        t->flags & MF_FRIEND && !t->player ? (*mapcolor_frnd_p) :
+        t->flags & MF_FRIEND && !t->player ? mapcolor_p->frnd :
         /* cph 2006/07/30 - Show count-as-kills in red. */
-        ((t->flags & (MF_COUNTKILL | MF_CORPSE)) == MF_COUNTKILL) ? (*mapcolor_enemy_p) :
+        ((t->flags & (MF_COUNTKILL | MF_CORPSE)) == MF_COUNTKILL) ? mapcolor_p->enemy :
         /* bbm 2/28/03 Show countable items in yellow. */
-        t->flags & MF_COUNTITEM ? (*mapcolor_item_p) : (*mapcolor_sprt_p),
+        t->flags & MF_COUNTITEM ? mapcolor_p->item : mapcolor_p->sprt,
         p.x, p.y);
       t = t->snext;
     }
@@ -2314,6 +2425,117 @@ static void AM_drawThings(void)
   }
 }
 
+void AM_updatePlayerTrail(fixed_t x, fixed_t y)
+{
+  trailpoint_t pt;
+
+  pt.x = x >> FRACTOMAPBITS;
+  pt.y = y >> FRACTOMAPBITS;
+
+  if (trail_index == -1 ||
+      player_trail[trail_index].x != pt.x ||
+      player_trail[trail_index].y != pt.y)
+  {
+    trail_index = (trail_index + 1) % trail_size_max;
+    player_trail[trail_index] = pt;
+
+    if (trail_size < trail_index + 1)
+      trail_size = trail_index + 1;
+  }
+}
+
+static void AM_drawPlayerTrail(void)
+{
+  int i;
+
+  for (i = 0; i < trail_size; ++i)
+  {
+    mpoint_t p;
+
+    p.x = player_trail[i].x;
+    p.y = player_trail[i].y;
+
+    if (automap_rotate)
+      AM_rotatePoint(&p);
+    else
+      AM_SetMPointFloatValue(&p);
+
+    p.x = CXMTOF(p.x);
+    p.y = CYMTOF(p.y);
+    if (am_frame.precise)
+    {
+      p.fx = CXMTOF_F(p.fx);
+      p.fy = CYMTOF_F(p.fy);
+    }
+
+    if (p.x >= f_x && p.y >= f_y && p.x < f_x + f_w && p.y < f_y + f_h)
+    {
+      mpoint_t a, b, c, d;
+      mpoint_t e, f, g, h;
+      mline_t line;
+
+      a.x = b.x = player_trail[i].x - FRACUNIT;
+      c.x = d.x = player_trail[i].x + FRACUNIT;
+      a.y = d.y = player_trail[i].y - FRACUNIT;
+      b.y = c.y = player_trail[i].y + FRACUNIT;
+
+      e.x = player_trail[i].x - FRACUNIT / 8;
+      e.y = player_trail[i].y - FRACUNIT / 8;
+      f.x = player_trail[i].x + FRACUNIT / 8;
+      f.y = player_trail[i].y + FRACUNIT / 8;
+      g.x = player_trail[i].x - FRACUNIT / 8;
+      g.y = player_trail[i].y + FRACUNIT / 8;
+      h.x = player_trail[i].x + FRACUNIT / 8;
+      h.y = player_trail[i].y - FRACUNIT / 8;
+
+      if (automap_rotate)
+      {
+        AM_rotatePoint(&a);
+        AM_rotatePoint(&b);
+        AM_rotatePoint(&c);
+        AM_rotatePoint(&d);
+        AM_rotatePoint(&e);
+        AM_rotatePoint(&f);
+        AM_rotatePoint(&g);
+        AM_rotatePoint(&h);
+      }
+      else
+      {
+        AM_SetMPointFloatValue(&a);
+        AM_SetMPointFloatValue(&b);
+        AM_SetMPointFloatValue(&c);
+        AM_SetMPointFloatValue(&d);
+        AM_SetMPointFloatValue(&e);
+        AM_SetMPointFloatValue(&f);
+        AM_SetMPointFloatValue(&g);
+        AM_SetMPointFloatValue(&h);
+      }
+
+      {
+        int color;
+
+        color = (i % 2) ? mapcolor_p->trail_1 : mapcolor_p->trail_2;
+
+        // Cross marking center
+        line.a = e; line.b = f;
+        AM_drawMline(&line, color);
+        line.a = g; line.b = h;
+        AM_drawMline(&line, color);
+
+        // Bounding box
+        line.a = a; line.b = b;
+        AM_drawMline(&line, color);
+        line.a = b; line.b = c;
+        AM_drawMline(&line, color);
+        line.a = c; line.b = d;
+        AM_drawMline(&line, color);
+        line.a = d; line.b = a;
+        AM_drawMline(&line, color);
+      }
+    }
+  }
+}
+
 //
 // AM_drawMarks()
 //
@@ -2329,6 +2551,9 @@ static void AM_drawMarks(void)
   int i;
   char namebuf[16] = "AMMNUM0";
 
+  if (map_trail_mode && dsda_RevealAutomap())
+    AM_drawPlayerTrail();
+
 #if defined(HAVE_LIBSDL2_IMAGE)
   if (V_IsOpenGLMode())
   {
@@ -2566,7 +2791,7 @@ void AM_Drawer (dboolean minimap)
   }
 
   if (!automap_overlay) // cph - If not overlay mode, clear background for the automap
-    V_FillRect(FB, f_x, f_y, f_w, f_h, (byte)(*mapcolor_back_p)); //jff 1/5/98 background default color
+    V_FillRect(FB, f_x, f_y, f_w, f_h, (byte)mapcolor_p->back); //jff 1/5/98 background default color
 
   if (map_textured)
   {
@@ -2576,11 +2801,12 @@ void AM_Drawer (dboolean minimap)
   }
 
   if (automap_grid)
-    AM_drawGrid((*mapcolor_grid_p));      //jff 1/7/98 grid default color
+    AM_drawGrid(mapcolor_p->grid);      //jff 1/7/98 grid default color
   AM_drawWalls();
   AM_drawPlayers();
   AM_drawThings(); //jff 1/5/98 default double IDDT sprite
-  AM_drawCrosshair((*mapcolor_hair_p));   //jff 1/7/98 default crosshair color
+  AM_DrawConnections();
+  AM_drawCrosshair(mapcolor_p->hair);   //jff 1/7/98 default crosshair color
 
   if (V_IsOpenGLMode())
   {
