@@ -73,7 +73,7 @@ typedef struct {
 #define CONF_COLOR(x) dsda_config_int, 0, 255, { x }
 #define CONF_BYTE(x) dsda_config_int, 0, 255, { x }
 #define CONF_STRING(x) dsda_config_string, 0, 0, { .v_string = x }
-#define CONF_CR(x) dsda_config_int, 0, CR_LIMIT - 1, { x }
+#define CONF_CR(x) dsda_config_int, 0, CR_HUD_LIMIT - 1, { x }
 #define CONF_WEAPON(x) dsda_config_int, 0, 9, { x }
 
 #define NOT_STRICT 0, 0
@@ -117,6 +117,7 @@ void dsda_InitParallelSFXFilter(void);
 void M_ChangeMapMultisamling(void);
 void M_ChangeMapTextured(void);
 void AM_InitParams(void);
+void AM_initPlayerTrail(void);
 void gld_ResetAutomapTransparency(void);
 void M_ChangeVideoMode(void);
 void M_ChangeUncappedFrameRate(void);
@@ -130,6 +131,7 @@ void dsda_InitGameControllerParameters(void);
 void dsda_InitExHud(void);
 void dsda_UpdateFreeText(void);
 void dsda_ResetAirControl(void);
+void dsda_AlterGameFlags(void);
 
 void dsda_TrackConfigFeatures(void) {
   if (!demorecording)
@@ -150,9 +152,6 @@ void dsda_TrackConfigFeatures(void) {
   if (dsda_IntConfig(dsda_config_coordinate_display) || dsda_IntConfig(dsda_config_map_coordinates))
     dsda_TrackFeature(uf_coordinates);
 
-  if (dsda_IntConfig(dsda_config_freelook))
-    dsda_TrackFeature(uf_mouselook);
-
   if (dsda_IntConfig(dsda_config_weapon_attack_alignment))
     dsda_TrackFeature(uf_weaponalignment);
 
@@ -322,7 +321,7 @@ dsda_config_t dsda_config[dsda_config_count] = {
   },
   [dsda_config_freelook] = {
     "allow_freelook", dsda_config_freelook,
-    CONF_BOOL(0), NULL, STRICT_INT(0), M_ChangeSkyMode
+    CONF_BOOL(0), NULL, NOT_STRICT, M_ChangeSkyMode
   },
   [dsda_config_autorun] = {
     "autorun", dsda_config_autorun,
@@ -368,6 +367,10 @@ dsda_config_t dsda_config[dsda_config_count] = {
     "dsda_mute_music", dsda_config_mute_music,
     CONF_BOOL(0), NULL, NOT_STRICT, I_ResetMusicVolume
   },
+  [dsda_config_mute_unfocused_window] = {
+    "dsda_mute_unfocused_window", dsda_config_mute_unfocused_window,
+    CONF_BOOL(0), NULL, NOT_STRICT, S_ResetVolume
+  },
   [dsda_config_cheat_codes] = {
     "dsda_cheat_codes", dsda_config_cheat_codes,
     CONF_BOOL(1)
@@ -476,109 +479,113 @@ dsda_config_t dsda_config[dsda_config_count] = {
     "comperr_blockmap", dsda_config_comperr_blockmap,
     CONF_BOOL(0), &default_comperr[comperr_blockmap]
   },
-  [dsda_config_comperr_freeaim] = {
-    "comperr_freeaim", dsda_config_comperr_freeaim,
-    CONF_BOOL(0), &default_comperr[comperr_freeaim]
-  },
   [dsda_config_mapcolor_back] = {
     "mapcolor_back", dsda_config_mapcolor_back,
-    CONF_COLOR(247), &mapcolor_back
+    CONF_COLOR(247), &mapcolor.back
   },
   [dsda_config_mapcolor_grid] = {
     "mapcolor_grid", dsda_config_mapcolor_grid,
-    CONF_COLOR(104), &mapcolor_grid
+    CONF_COLOR(104), &mapcolor.grid
   },
   [dsda_config_mapcolor_wall] = {
     "mapcolor_wall", dsda_config_mapcolor_wall,
-    CONF_COLOR(23), &mapcolor_wall
+    CONF_COLOR(23), &mapcolor.wall
   },
   [dsda_config_mapcolor_fchg] = {
     "mapcolor_fchg", dsda_config_mapcolor_fchg,
-    CONF_COLOR(55), &mapcolor_fchg
+    CONF_COLOR(55), &mapcolor.fchg
   },
   [dsda_config_mapcolor_cchg] = {
     "mapcolor_cchg", dsda_config_mapcolor_cchg,
-    CONF_COLOR(215), &mapcolor_cchg
+    CONF_COLOR(215), &mapcolor.cchg
   },
   [dsda_config_mapcolor_clsd] = {
     "mapcolor_clsd", dsda_config_mapcolor_clsd,
-    CONF_COLOR(208), &mapcolor_clsd
+    CONF_COLOR(208), &mapcolor.clsd
   },
   [dsda_config_mapcolor_rkey] = {
     "mapcolor_rkey", dsda_config_mapcolor_rkey,
-    CONF_COLOR(175), &mapcolor_rkey
+    CONF_COLOR(175), &mapcolor.rkey
   },
   [dsda_config_mapcolor_bkey] = {
     "mapcolor_bkey", dsda_config_mapcolor_bkey,
-    CONF_COLOR(204), &mapcolor_bkey
+    CONF_COLOR(204), &mapcolor.bkey
   },
   [dsda_config_mapcolor_ykey] = {
     "mapcolor_ykey", dsda_config_mapcolor_ykey,
-    CONF_COLOR(231), &mapcolor_ykey
+    CONF_COLOR(231), &mapcolor.ykey
   },
   [dsda_config_mapcolor_rdor] = {
     "mapcolor_rdor", dsda_config_mapcolor_rdor,
-    CONF_COLOR(175), &mapcolor_rdor
+    CONF_COLOR(175), &mapcolor.rdor
   },
   [dsda_config_mapcolor_bdor] = {
     "mapcolor_bdor", dsda_config_mapcolor_bdor,
-    CONF_COLOR(204), &mapcolor_bdor
+    CONF_COLOR(204), &mapcolor.bdor
   },
   [dsda_config_mapcolor_ydor] = {
     "mapcolor_ydor", dsda_config_mapcolor_ydor,
-    CONF_COLOR(231), &mapcolor_ydor
+    CONF_COLOR(231), &mapcolor.ydor
   },
   [dsda_config_mapcolor_tele] = {
     "mapcolor_tele", dsda_config_mapcolor_tele,
-    CONF_COLOR(119), &mapcolor_tele
+    CONF_COLOR(119), &mapcolor.tele
   },
   [dsda_config_mapcolor_secr] = {
     "mapcolor_secr", dsda_config_mapcolor_secr,
-    CONF_COLOR(252), &mapcolor_secr
+    CONF_COLOR(252), &mapcolor.secr
   },
   [dsda_config_mapcolor_revsecr] = {
     "mapcolor_revsecr", dsda_config_mapcolor_revsecr,
-    CONF_COLOR(112), &mapcolor_revsecr
+    CONF_COLOR(112), &mapcolor.revsecr
   },
   [dsda_config_mapcolor_exit] = {
     "mapcolor_exit", dsda_config_mapcolor_exit,
-    CONF_COLOR(0), &mapcolor_exit
+    CONF_COLOR(0), &mapcolor.exit
   },
   [dsda_config_mapcolor_unsn] = {
     "mapcolor_unsn", dsda_config_mapcolor_unsn,
-    CONF_COLOR(104), &mapcolor_unsn
+    CONF_COLOR(104), &mapcolor.unsn
   },
   [dsda_config_mapcolor_flat] = {
     "mapcolor_flat", dsda_config_mapcolor_flat,
-    CONF_COLOR(88), &mapcolor_flat
+    CONF_COLOR(88), &mapcolor.flat
   },
   [dsda_config_mapcolor_sprt] = {
     "mapcolor_sprt", dsda_config_mapcolor_sprt,
-    CONF_COLOR(112), &mapcolor_sprt
+    CONF_COLOR(112), &mapcolor.sprt
   },
   [dsda_config_mapcolor_item] = {
     "mapcolor_item", dsda_config_mapcolor_item,
-    CONF_COLOR(231), &mapcolor_item
+    CONF_COLOR(231), &mapcolor.item
   },
   [dsda_config_mapcolor_hair] = {
     "mapcolor_hair", dsda_config_mapcolor_hair,
-    CONF_COLOR(208), &mapcolor_hair
+    CONF_COLOR(208), &mapcolor.hair
   },
   [dsda_config_mapcolor_sngl] = {
     "mapcolor_sngl", dsda_config_mapcolor_sngl,
-    CONF_COLOR(208), &mapcolor_sngl
+    CONF_COLOR(208), &mapcolor.sngl
   },
   [dsda_config_mapcolor_me] = {
     "mapcolor_me", dsda_config_mapcolor_me,
-    CONF_COLOR(112), &mapcolor_me
+    CONF_COLOR(112), &mapcolor.me
   },
   [dsda_config_mapcolor_enemy] = {
     "mapcolor_enemy", dsda_config_mapcolor_enemy,
-    CONF_COLOR(177), &mapcolor_enemy
+    CONF_COLOR(177), &mapcolor.enemy
   },
   [dsda_config_mapcolor_frnd] = {
     "mapcolor_frnd", dsda_config_mapcolor_frnd,
-    CONF_COLOR(112), &mapcolor_frnd
+    CONF_COLOR(112), &mapcolor.frnd
+  },
+  [dsda_config_mapcolor_trail_1] = {
+    "mapcolor_trail_1", dsda_config_mapcolor_trail_1,
+    CONF_COLOR(80), &mapcolor.trail_1
+  },
+  [dsda_config_mapcolor_trail_2] = {
+    "mapcolor_trail_2", dsda_config_mapcolor_trail_2,
+    CONF_COLOR(100), &mapcolor.trail_2
   },
   [dsda_config_gl_skymode] = {
     "gl_skymode", dsda_config_gl_skymode,
@@ -889,6 +896,10 @@ dsda_config_t dsda_config[dsda_config_count] = {
     "dsda_auto_key_frame_timeout", dsda_config_auto_key_frame_timeout,
     dsda_config_int, 0, 25, { 10 }, NULL, NOT_STRICT, dsda_InitKeyFrame
   },
+  [dsda_config_auto_save] = {
+    "dsda_config_auto_save", dsda_config_auto_save,
+    CONF_BOOL(0)
+  },
   [dsda_config_ex_text_scale_x] = {
     "ex_text_scale_x", dsda_config_ex_text_scale_x,
     dsda_config_int, 0, 4000, { 0 }, NULL, NOT_STRICT, dsda_SetupStretchParams
@@ -949,6 +960,26 @@ dsda_config_t dsda_config[dsda_config_count] = {
     "dsda_allow_jumping", dsda_config_allow_jumping,
     CONF_BOOL(0), NULL, NOT_STRICT, dsda_ResetAirControl
   },
+  [dsda_config_pistol_start] = {
+    "dsda_pistol_start", dsda_config_pistol_start,
+    CONF_BOOL(0)
+  },
+  [dsda_config_respawn_monsters] = {
+    "dsda_respawn_monsters", dsda_config_respawn_monsters,
+    CONF_BOOL(0), NULL, NOT_STRICT, dsda_AlterGameFlags
+  },
+  [dsda_config_fast_monsters] = {
+    "dsda_fast_monsters", dsda_config_fast_monsters,
+    CONF_BOOL(0), NULL, NOT_STRICT, dsda_AlterGameFlags
+  },
+  [dsda_config_no_monsters] = {
+    "dsda_no_monsters", dsda_config_no_monsters,
+    CONF_BOOL(0), NULL, NOT_STRICT, dsda_AlterGameFlags
+  },
+  [dsda_config_coop_spawns] = {
+    "dsda_coop_spawns", dsda_config_coop_spawns,
+    CONF_BOOL(0), NULL, NOT_STRICT, dsda_AlterGameFlags
+  },
   [dsda_config_parallel_sfx_limit] = {
     "dsda_parallel_sfx_limit", dsda_config_parallel_sfx_limit,
     dsda_config_int, 0, 32, { 0 }, NULL, NOT_STRICT, dsda_InitParallelSFXFilter
@@ -1001,6 +1032,14 @@ dsda_config_t dsda_config[dsda_config_count] = {
     "map_title", dsda_config_map_title,
     CONF_BOOL(1), NULL, NOT_STRICT, dsda_RefreshMapTitle
   },
+  [dsda_config_map_trail_mode] = {
+    "map_trail_mode", dsda_config_map_trail_mode,
+    dsda_config_int, 0, map_trail_mode_max - 1, { 0 }, NULL, NOT_STRICT, AM_initPlayerTrail
+  },
+  [dsda_config_map_trail_size] = {
+    "map_trail_size", dsda_config_map_trail_size,
+    dsda_config_int, 0, 350, { 105 }, NULL, NOT_STRICT, AM_initPlayerTrail
+  },
   [dsda_config_automap_overlay] = {
     "automap_overlay", dsda_config_automap_overlay,
     CONF_BOOL(0), &automap_overlay
@@ -1216,6 +1255,10 @@ dsda_config_t dsda_config[dsda_config_count] = {
     "ansi_endoom", dsda_config_ansi_endoom,
     dsda_config_int, 0, 2, { 0 }
   },
+  [dsda_config_quit_sounds] = {
+    "quit_sounds", dsda_config_quit_sounds,
+    CONF_BOOL(0),
+  },
   [dsda_config_announce_map] = {
     "announce_map", dsda_config_announce_map,
     CONF_BOOL(0),
