@@ -95,8 +95,6 @@ const int tran_filter_pct = 66;
 GLfloat gl_texture_filter_anisotropic;
 
 extern int gld_paletteIndex;
-extern int playpal_black;
-extern int playpal_white;
 
 //sprites
 const float gl_spriteclip_threshold_f = 10.f / MAP_COEFF;
@@ -591,15 +589,6 @@ void gld_FillRaw(int lump, int x, int y, int src_width, int src_height, int dst_
   if (!gltexture)
     return;
 
-  // [XA] NOTE: this flag actually means "tile", not stretch...
-  if (flags & VPT_STRETCH)
-  {
-    x = x * SCREENWIDTH / 320;
-    y = y * SCREENHEIGHT / 200;
-    dst_width = dst_width * SCREENWIDTH / 320;
-    dst_height = dst_height * SCREENHEIGHT / 200;
-  }
-
   fU1 = 0;
   fV1 = 0;
 
@@ -609,6 +598,13 @@ void gld_FillRaw(int lump, int x, int y, int src_width, int src_height, int dst_
     fU2 = 1.0f;
     fV2 = 1.0f;
   }
+  else if (flags & VPT_STRETCH)
+  {
+    stretch_param_t *params = dsda_StretchParams(flags);
+
+    fU2 = (float)dst_width / (float)gltexture->realtexwidth / (params->video->width / 320.f);
+    fV2 = (float)dst_height / (float)gltexture->realtexheight / (params->video->height / 200.f);
+  }
   else
   {
     fU2 = (float)dst_width / (float)gltexture->realtexwidth;
@@ -1177,8 +1173,8 @@ static void gld_DrawWall(GLWall *wall)
 static void gld_CalculateWallY(GLWall *wall, float *lineheight,
                                fixed_t floor_height, fixed_t ceiling_height)
 {
-  wall->ytop = (float) ceiling_height / (float) MAP_SCALE + SMALLDELTA;
-  wall->ybottom = (float) floor_height / (float) MAP_SCALE - SMALLDELTA;
+  wall->ytop = (float) ceiling_height / (float) MAP_SCALE;
+  wall->ybottom = (float) floor_height / (float) MAP_SCALE;
   *lineheight = (float) fabs((float) (ceiling_height - floor_height) / FRACUNIT);
 }
 
@@ -1288,14 +1284,14 @@ void gld_AddWall(seg_t *seg)
 
     if (frontsector->ceilingpic==skyflatnum)
     {
-      wall.ytop=MAXCOORD;
+      wall.ytop=MAXCOORD*2; // Simply using MAXCOORD would result in HOM when the floor is at a height close to the limit
       wall.ybottom=(float)frontsector->ceilingheight/MAP_SCALE;
       gld_AddSkyTexture(&wall, frontsector->ceilingsky, frontsector->ceilingsky, SKY_CEILING);
     }
     if (frontsector->floorpic==skyflatnum)
     {
       wall.ytop=(float)frontsector->floorheight/MAP_SCALE;
-      wall.ybottom=-MAXCOORD;
+      wall.ybottom=-MAXCOORD*2;  // Simply using MAXCOORD would result in HOM when the ceiling is at a height close to the limit
       gld_AddSkyTexture(&wall, frontsector->floorsky, frontsector->floorsky, SKY_FLOOR);
     }
     temptex=gld_RegisterTexture(texturetranslation[seg->sidedef->midtexture], true, false, true, false);
@@ -1368,7 +1364,7 @@ void gld_AddWall(seg_t *seg)
     wall.yscale = (float) seg->sidedef->scaley_top / FRACUNIT;
     if (frontsector->ceilingpic==skyflatnum)// || backsector->ceilingpic==skyflatnum)
     {
-      wall.ytop= MAXCOORD;
+      wall.ytop= MAXCOORD*2;
       if (
           // e6y
           // There is no more HOM in the starting area on Memento Mori map29 and on map30.
@@ -1433,8 +1429,8 @@ void gld_AddWall(seg_t *seg)
           !(backsector->flags & NULL_SECTOR) &&
           backsector->floorheight < backsector->ceilingheight)
         {
-          wall.ytop=((float)(ceiling_height)/(float)MAP_SCALE)+SMALLDELTA;
-          wall.ybottom=((float)(floor_height)/(float)MAP_SCALE)-SMALLDELTA;
+          wall.ytop=((float)(ceiling_height)/(float)MAP_SCALE);
+          wall.ybottom=((float)(floor_height)/(float)MAP_SCALE);
           if (wall.ybottom >= zCamera)
           {
             wall.flag=GLDWF_TOPFLUD;
@@ -1602,10 +1598,11 @@ bottomtexture:
     wall.yscale = (float) seg->sidedef->scaley_bottom / FRACUNIT;
     if (frontsector->floorpic==skyflatnum)
     {
-      wall.ybottom=-MAXCOORD;
+      wall.ybottom=-MAXCOORD*2;
       if (
           (backsector->ceilingheight==backsector->floorheight) &&
-          (backsector->floorpic==skyflatnum)
+          (backsector->floorpic==skyflatnum) &&
+          (bottomtexture == NO_TEXTURE)
          )
       {
         wall.ytop=(float)backsector->floorheight/MAP_SCALE;
@@ -1639,8 +1636,8 @@ bottomtexture:
         !(backsector->flags & NULL_SECTOR) &&
         backsector->floorheight < backsector->ceilingheight)
       {
-        wall.ytop=((float)(ceiling_height)/(float)MAP_SCALE)+SMALLDELTA;
-        wall.ybottom=((float)(floor_height)/(float)MAP_SCALE)-SMALLDELTA;
+        wall.ytop=((float)(ceiling_height)/(float)MAP_SCALE);
+        wall.ybottom=((float)(floor_height)/(float)MAP_SCALE);
         if (wall.ytop <= zCamera)
         {
           wall.flag = GLDWF_BOTFLUD;
